@@ -34,7 +34,7 @@ class WP_Roles {
 	 * @access public
 	 * @var array
 	 */
-	var $roles;
+	public $roles;
 
 	/**
 	 * List of the role objects.
@@ -43,7 +43,7 @@ class WP_Roles {
 	 * @access public
 	 * @var array
 	 */
-	var $role_objects = array();
+	public $role_objects = array();
 
 	/**
 	 * List of role names.
@@ -52,7 +52,7 @@ class WP_Roles {
 	 * @access public
 	 * @var array
 	 */
-	var $role_names = array();
+	public $role_names = array();
 
 	/**
 	 * Option name for storing role list.
@@ -61,7 +61,7 @@ class WP_Roles {
 	 * @access public
 	 * @var string
 	 */
-	var $role_key;
+	public $role_key;
 
 	/**
 	 * Whether to use the database for retrieval and storage.
@@ -70,17 +70,31 @@ class WP_Roles {
 	 * @access public
 	 * @var bool
 	 */
-	var $use_db = true;
+	public $use_db = true;
 
 	/**
 	 * Constructor
 	 *
 	 * @since 2.0.0
 	 */
-	function __construct() {
+	public function __construct() {
 		$this->_init();
 	}
 
+	/**
+	 * Make private/protected methods readable for backwards compatibility.
+	 *
+	 * @since 4.0.0
+	 * @access public
+	 *
+	 * @param callable $name      Method to call.
+	 * @param array    $arguments Arguments to pass when calling.
+	 * @return mixed|bool Return value of the callback, false otherwise.
+	 */
+	public function __call( $name, $arguments ) {
+		return call_user_func_array( array( $this, $name ), $arguments );
+	}
+
 	/**
 	 * Set up the object properties.
 	 *
@@ -93,7 +107,7 @@ class WP_Roles {
 	 * @uses $wpdb Used to get the database prefix.
 	 * @global array $wp_user_roles Used to set the 'roles' property value.
 	 */
-	function _init () {
+	protected function _init() {
 		global $wpdb, $wp_user_roles;
 		$this->role_key = $wpdb->get_blog_prefix() . 'user_roles';
 		if ( ! empty( $wp_user_roles ) ) {
@@ -123,12 +137,12 @@ class WP_Roles {
 	 * @since 3.5.0
 	 * @access public
 	 */
-	function reinit() {
+	public function reinit() {
 		// There is no need to reinit if using the wp_user_roles global.
 		if ( ! $this->use_db )
 			return;
 
-		global $wpdb, $wp_user_roles;
+		global $wpdb;
 
 		// Duplicated from _init() to avoid an extra function call.
 		$this->role_key = $wpdb->get_blog_prefix() . 'user_roles';
@@ -160,7 +174,7 @@ class WP_Roles {
 	 * @param array $capabilities List of role capabilities in the above format.
 	 * @return WP_Role|null WP_Role object if role is added, null if already exists.
 	 */
-	function add_role( $role, $display_name, $capabilities = array() ) {
+	public function add_role( $role, $display_name, $capabilities = array() ) {
 		if ( isset( $this->roles[$role] ) )
 			return;
 
@@ -183,7 +197,7 @@ class WP_Roles {
 	 *
 	 * @param string $role Role name.
 	 */
-	function remove_role( $role ) {
+	public function remove_role( $role ) {
 		if ( ! isset( $this->role_objects[$role] ) )
 			return;
 
@@ -208,7 +222,7 @@ class WP_Roles {
 	 * @param string $cap Capability name.
 	 * @param bool $grant Optional, default is true. Whether role is capable of performing capability.
 	 */
-	function add_cap( $role, $cap, $grant = true ) {
+	public function add_cap( $role, $cap, $grant = true ) {
 		if ( ! isset( $this->roles[$role] ) )
 			return;
 
@@ -226,7 +240,7 @@ class WP_Roles {
 	 * @param string $role Role name.
 	 * @param string $cap Capability name.
 	 */
-	function remove_cap( $role, $cap ) {
+	public function remove_cap( $role, $cap ) {
 		if ( ! isset( $this->roles[$role] ) )
 			return;
 
@@ -244,7 +258,7 @@ class WP_Roles {
 	 * @param string $role Role name.
 	 * @return WP_Role|null WP_Role object if found, null if the role does not exist.
 	 */
-	function get_role( $role ) {
+	public function get_role( $role ) {
 		if ( isset( $this->role_objects[$role] ) )
 			return $this->role_objects[$role];
 		else
@@ -259,7 +273,7 @@ class WP_Roles {
 	 *
 	 * @return array List of role names.
 	 */
-	function get_names() {
+	public function get_names() {
 		return $this->role_names;
 	}
 
@@ -272,7 +286,7 @@ class WP_Roles {
 	 * @param string $role Role name to look up.
 	 * @return bool
 	 */
-	function is_role( $role ) {
+	public function is_role( $role ) {
 		return isset( $this->role_names[$role] );
 	}
 }
@@ -292,7 +306,7 @@ class WP_Role {
 	 * @access public
 	 * @var string
 	 */
-	var $name;
+	public $name;
 
 	/**
 	 * List of capabilities the role contains.
@@ -301,7 +315,7 @@ class WP_Role {
 	 * @access public
 	 * @var array
 	 */
-	var $capabilities;
+	public $capabilities;
 
 	/**
 	 * Constructor - Set up object properties.
@@ -315,7 +329,7 @@ class WP_Role {
 	 * @param string $role Role name.
 	 * @param array $capabilities List of capabilities.
 	 */
-	function __construct( $role, $capabilities ) {
+	public function __construct( $role, $capabilities ) {
 		$this->name = $role;
 		$this->capabilities = $capabilities;
 	}
@@ -330,7 +344,7 @@ class WP_Role {
 	 * @param string $cap Capability name.
 	 * @param bool $grant Whether role has capability privilege.
 	 */
-	function add_cap( $cap, $grant = true ) {
+	public function add_cap( $cap, $grant = true ) {
 		global $wp_roles;
 
 		if ( ! isset( $wp_roles ) )
@@ -353,7 +367,7 @@ class WP_Role {
 	 *
 	 * @param string $cap Capability name.
 	 */
-	function remove_cap( $cap ) {
+	public function remove_cap( $cap ) {
 		global $wp_roles;
 
 		if ( ! isset( $wp_roles ) )
@@ -377,7 +391,7 @@ class WP_Role {
 	 * @param string $cap Capability name.
 	 * @return bool True, if user has capability. False, if doesn't have capability.
 	 */
-	function has_cap( $cap ) {
+	public function has_cap( $cap ) {
 		/**
 		 * Filter which capabilities a role has.
 		 *
@@ -420,7 +434,7 @@ class WP_User {
 	 * @access public
 	 * @var int
 	 */
-	var $ID = 0;
+	public $ID = 0;
 
 	/**
 	 * The individual capabilities the user has been given.
@@ -429,7 +443,7 @@ class WP_User {
 	 * @access public
 	 * @var array
 	 */
-	var $caps = array();
+	public $caps = array();
 
 	/**
 	 * User metadata option name.
@@ -438,7 +452,7 @@ class WP_User {
 	 * @access public
 	 * @var string
 	 */
-	var $cap_key;
+	public $cap_key;
 
 	/**
 	 * The roles the user is part of.
@@ -447,7 +461,7 @@ class WP_User {
 	 * @access public
 	 * @var array
 	 */
-	var $roles = array();
+	public $roles = array();
 
 	/**
 	 * All capabilities the user has, including individual and role based.
@@ -456,7 +470,7 @@ class WP_User {
 	 * @access public
 	 * @var array
 	 */
-	var $allcaps = array();
+	public $allcaps = array();
 
 	/**
 	 * The filter context applied to user data fields.
@@ -482,7 +496,7 @@ class WP_User {
 	 * @param int $blog_id Optional Blog ID, defaults to current blog.
 	 * @return WP_User
 	 */
-	function __construct( $id = 0, $name = '', $blog_id = '' ) {
+	public function __construct( $id = 0, $name = '', $blog_id = '' ) {
 		if ( ! isset( self::$back_compat_keys ) ) {
 			$prefix = $GLOBALS['wpdb']->prefix;
 			self::$back_compat_keys = array(
@@ -523,7 +537,7 @@ class WP_User {
 	 * @param object $data User DB row object
 	 * @param int $blog_id Optional. The blog id to initialize for
 	 */
-	function init( $data, $blog_id = '' ) {
+	public function init( $data, $blog_id = '' ) {
 		$this->data = $data;
 		$this->ID = (int) $data->ID;
 
@@ -539,7 +553,7 @@ class WP_User {
 	 * @param string|int $value The field value
 	 * @return object Raw user object
 	 */
-	static function get_data_by( $field, $value ) {
+	public static function get_data_by( $field, $value ) {
 		global $wpdb;
 
 		if ( 'id' == $field ) {
@@ -599,7 +613,7 @@ class WP_User {
 	 *
 	 * @since 3.3.0
 	 */
-	function __isset( $key ) {
+	public function __isset( $key ) {
 		if ( 'id' == $key ) {
 			_deprecated_argument( 'WP_User->id', '2.1', __( 'Use <code>WP_User->ID</code> instead.' ) );
 			$key = 'ID';
@@ -619,7 +633,7 @@ class WP_User {
 	 *
 	 * @since 3.3.0
 	 */
-	function __get( $key ) {
+	public function __get( $key ) {
 		if ( 'id' == $key ) {
 			_deprecated_argument( 'WP_User->id', '2.1', __( 'Use <code>WP_User->ID</code> instead.' ) );
 			return $this->ID;
@@ -645,7 +659,7 @@ class WP_User {
 	 *
 	 * @since 3.3.0
 	 */
-	function __set( $key, $value ) {
+	public function __set( $key, $value ) {
 		if ( 'id' == $key ) {
 			_deprecated_argument( 'WP_User->id', '2.1', __( 'Use <code>WP_User->ID</code> instead.' ) );
 			$this->ID = $value;
@@ -663,7 +677,7 @@ class WP_User {
 	 *
 	 * @return bool True if user exists in the database, false if not.
 	 */
-	function exists() {
+	public function exists() {
 		return ! empty( $this->ID );
 	}
 
@@ -676,7 +690,7 @@ class WP_User {
 	 *
 	 * @param string $key Property
 	 */
-	function get( $key ) {
+	public function get( $key ) {
 		return $this->__get( $key );
 	}
 
@@ -689,7 +703,7 @@ class WP_User {
 	 *
 	 * @param string $key Property
 	 */
-	function has_prop( $key ) {
+	public function has_prop( $key ) {
 		return $this->__isset( $key );
 	}
 
@@ -700,7 +714,7 @@ class WP_User {
 	 *
 	 * @return array Array representation.
 	 */
-	function to_array() {
+	public function to_array() {
 		return get_object_vars( $this->data );
 	}
 
@@ -747,7 +761,7 @@ class WP_User {
 	 *
 	 * @return array List of all capabilities for the user.
 	 */
-	function get_role_caps() {
+	public function get_role_caps() {
 		global $wp_roles;
 
 		if ( ! isset( $wp_roles ) )
@@ -778,7 +792,7 @@ class WP_User {
 	 *
 	 * @param string $role Role name.
 	 */
-	function add_role( $role ) {
+	public function add_role( $role ) {
 		$this->caps[$role] = true;
 		update_user_meta( $this->ID, $this->cap_key, $this->caps );
 		$this->get_role_caps();
@@ -793,7 +807,7 @@ class WP_User {
 	 *
 	 * @param string $role Role name.
 	 */
-	function remove_role( $role ) {
+	public function remove_role( $role ) {
 		if ( !in_array($role, $this->roles) )
 			return;
 		unset( $this->caps[$role] );
@@ -814,7 +828,7 @@ class WP_User {
 	 *
 	 * @param string $role Role name.
 	 */
-	function set_role( $role ) {
+	public function set_role( $role ) {
 		if ( 1 == count( $this->roles ) && $role == current( $this->roles ) )
 			return;
 
@@ -864,7 +878,7 @@ class WP_User {
 	 * @param string $item Level capability name.
 	 * @return int Max Level.
 	 */
-	function level_reduction( $max, $item ) {
+	public function level_reduction( $max, $item ) {
 		if ( preg_match( '/^level_(10|[0-9])$/i', $item, $matches ) ) {
 			$level = intval( $matches[1] );
 			return max( $max, $level );
@@ -883,7 +897,7 @@ class WP_User {
 	 * @since 2.0.0
 	 * @access public
 	 */
-	function update_user_level_from_caps() {
+	public function update_user_level_from_caps() {
 		global $wpdb;
 		$this->user_level = array_reduce( array_keys( $this->allcaps ), array( $this, 'level_reduction' ), 0 );
 		update_user_meta( $this->ID, $wpdb->get_blog_prefix() . 'user_level', $this->user_level );
@@ -898,7 +912,7 @@ class WP_User {
 	 * @param string $cap Capability name.
 	 * @param bool $grant Whether to grant capability to user.
 	 */
-	function add_cap( $cap, $grant = true ) {
+	public function add_cap( $cap, $grant = true ) {
 		$this->caps[$cap] = $grant;
 		update_user_meta( $this->ID, $this->cap_key, $this->caps );
 	}
@@ -911,7 +925,7 @@ class WP_User {
 	 *
 	 * @param string $cap Capability name.
 	 */
-	function remove_cap( $cap ) {
+	public function remove_cap( $cap ) {
 		if ( ! isset( $this->caps[$cap] ) )
 			return;
 		unset( $this->caps[$cap] );
@@ -924,7 +938,7 @@ class WP_User {
 	 * @since 2.1.0
 	 * @access public
 	 */
-	function remove_all_caps() {
+	public function remove_all_caps() {
 		global $wpdb;
 		$this->caps = array();
 		delete_user_meta( $this->ID, $this->cap_key );
@@ -945,7 +959,7 @@ class WP_User {
 	 * @param string|int $cap Capability or role name to search.
 	 * @return bool True, if user has capability; false, if user does not have capability.
 	 */
-	function has_cap( $cap ) {
+	public function has_cap( $cap ) {
 		if ( is_numeric( $cap ) ) {
 			_deprecated_argument( __FUNCTION__, '2.0', __('Usage of user levels by plugins and themes is deprecated. Use roles and capabilities instead.') );
 			$cap = $this->translate_level_to_cap( $cap );
@@ -995,7 +1009,7 @@ class WP_User {
 	 * @param int $level Level number, 1 to 10.
 	 * @return string
 	 */
-	function translate_level_to_cap( $level ) {
+	public function translate_level_to_cap( $level ) {
 		return 'level_' . $level;
 	}
 
@@ -1006,7 +1020,7 @@ class WP_User {
 	 *
 	 * @param int $blog_id Optional Blog ID, defaults to current blog.
 	 */
-	function for_blog( $blog_id = '' ) {
+	public function for_blog( $blog_id = '' ) {
 		global $wpdb;
 		if ( ! empty( $blog_id ) )
 			$cap_key = $wpdb->get_blog_prefix( $blog_id ) . 'capabilities';
@@ -1248,18 +1262,25 @@ function map_meta_cap( $cap, $user_id ) {
 	case 'update_plugins':
 	case 'delete_plugins':
 	case 'install_plugins':
+	case 'upload_plugins':
 	case 'update_themes':
 	case 'delete_themes':
 	case 'install_themes':
+	case 'upload_themes':
 	case 'update_core':
 		// Disallow anything that creates, deletes, or updates core, plugin, or theme files.
 		// Files in uploads are excepted.
-		if ( defined( 'DISALLOW_FILE_MODS' ) && DISALLOW_FILE_MODS )
+		if ( defined( 'DISALLOW_FILE_MODS' ) && DISALLOW_FILE_MODS ) {
 			$caps[] = 'do_not_allow';
-		elseif ( is_multisite() && ! is_super_admin( $user_id ) )
+		} elseif ( is_multisite() && ! is_super_admin( $user_id ) ) {
 			$caps[] = 'do_not_allow';
-		else
+		} elseif ( 'upload_themes' === $cap ) {
+			$caps[] = 'install_themes';
+		} elseif ( 'upload_plugins' === $cap ) {
+			$caps[] = 'install_plugins';
+		} else {
 			$caps[] = $cap;
+		}
 		break;
 	case 'activate_plugins':
 		$caps[] = $cap;
@@ -1292,6 +1313,9 @@ function map_meta_cap( $cap, $user_id ) {
 		else
 			$caps[] = 'do_not_allow';
 		break;
+	case 'customize' :
+		$caps[] = 'edit_theme_options';
+		break;
 	default:
 		// Handle meta capabilities for custom post types.
 		$post_type_meta_caps = _post_type_meta_capabilities();
@@ -1347,25 +1371,21 @@ function current_user_can( $capability ) {
  * @return bool
  */
 function current_user_can_for_blog( $blog_id, $capability ) {
-	$switched = is_multisite() ? switch_to_blog( $blog_id ) : false;
+	if ( is_multisite() )
+		switch_to_blog( $blog_id );
 
 	$current_user = wp_get_current_user();
 
-	if ( empty( $current_user ) ) {
-		if ( $switched ) {
-			restore_current_blog();
-		}
+	if ( empty( $current_user ) )
 		return false;
-	}
 
 	$args = array_slice( func_get_args(), 2 );
 	$args = array_merge( array( $capability ), $args );
 
 	$can = call_user_func_array( array( $current_user, 'has_cap' ), $args );
 
-	if ( $switched ) {
+	if ( is_multisite() )
 		restore_current_blog();
-	}
 
 	return $can;
 }
